@@ -1,10 +1,11 @@
 #include <config.h>
 
-#include "rcsid.h"
-RCSID ("$Id: pwdcheck.c,v 1.3 2004/06/02 23:50:10 kloczek Exp $")
+#ident "$Id: pwdcheck.c,v 1.8 2005/08/31 17:24:58 kloczek Exp $"
+
+#include <pwd.h>
+#include <stdio.h>
 #include "prototypes.h"
 #include "defines.h"
-#include <pwd.h>
 #include "pwauth.h"
 #ifdef HAVE_SHADOW_H
 #include <shadow.h>
@@ -13,8 +14,7 @@ RCSID ("$Id: pwdcheck.c,v 1.3 2004/06/02 23:50:10 kloczek Exp $")
 #include "pam_defs.h"
 #endif
 #define WRONGPWD2	"incorrect password for `%s'"
-void
-passwd_check (const char *user, const char *passwd, const char *progname)
+void passwd_check (const char *user, const char *passwd, const char *progname)
 {
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
@@ -34,7 +34,7 @@ passwd_check (const char *user, const char *passwd, const char *progname)
 	retcode = pam_acct_mgmt (pamh, 0);
 	if (retcode == PAM_NEW_AUTHTOK_REQD)
 		retcode = pam_chauthtok (pamh, PAM_CHANGE_EXPIRED_AUTHTOK);
-	else if (retcode)
+	if (retcode)
 		goto bailout;
 
 	if (pam_setcred (pamh, 0))
@@ -45,13 +45,11 @@ passwd_check (const char *user, const char *passwd, const char *progname)
 
 #else				/* !USE_PAM */
 
-#ifdef SHADOWPWD
 	struct spwd *sp;
 
 	if ((sp = getspnam (user)))
 		passwd = sp->sp_pwdp;
 	endspent ();
-#endif
 	if (pw_auth (passwd, user, PW_LOGIN, (char *) 0) != 0) {
 		SYSLOG ((LOG_WARN, WRONGPWD2, user));
 		sleep (1);
