@@ -37,12 +37,12 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: id.c,v 1.13 2003/06/19 18:11:01 kloczek Exp $")
-#include <sys/types.h>
-#include <stdio.h>
+#ident "$Id: id.c,v 1.18 2005/09/07 15:00:45 kloczek Exp $"
+
 #include <grp.h>
 #include <pwd.h>
+#include <stdio.h>
+#include <sys/types.h>
 #include "defines.h"
 /* local function prototypes */
 static void usage (void);
@@ -120,15 +120,15 @@ static void usage (void)
 
 	pw = getpwuid (ruid);
 	if (pw)
-		printf ("uid=%u(%s)", ruid, pw->pw_name);
+		printf ("UID=%u(%s)", ruid, pw->pw_name);
 	else
-		printf ("uid=%u", ruid);
+		printf ("UID=%u", ruid);
 
 	gr = getgrgid (rgid);
 	if (gr)
-		printf (" gid=%u(%s)", rgid, gr->gr_name);
+		printf (" GID=%u(%s)", rgid, gr->gr_name);
 	else
-		printf (" gid=%u", rgid);
+		printf (" GID=%u", rgid);
 
 	/*
 	 * Print out the effective user ID and group ID if they are
@@ -138,23 +138,22 @@ static void usage (void)
 	if (ruid != euid) {
 		pw = getpwuid (euid);
 		if (pw)
-			printf (" euid=%u(%s)", euid, pw->pw_name);
+			printf (" EUID=%u(%s)", euid, pw->pw_name);
 		else
-			printf (" euid=%u", euid);
+			printf (" EUID=%u", euid);
 	}
 	if (rgid != egid) {
 		gr = getgrgid (egid);
 		if (gr)
-			printf (" egid=%u(%s)", egid, gr->gr_name);
+			printf (" EGID=%u(%s)", egid, gr->gr_name);
 		else
-			printf (" egid=%u", egid);
+			printf (" EGID=%u", egid);
 	}
 #ifdef HAVE_GETGROUPS
 	/*
 	 * Print out the concurrent group set if the user has requested it.
 	 * The group numbers will be printed followed by their names.
 	 */
-
 	if (aflg && (ngroups = getgroups (sys_ngroups, groups)) != -1) {
 
 		/*
@@ -165,7 +164,6 @@ static void usage (void)
 		 * where "###" is a numerical value and "aaa" is the
 		 * corresponding name for each respective numerical value.
 		 */
-
 		printf (_(" groups="));
 		for (i = 0; i < ngroups; i++) {
 			if (i)
@@ -184,7 +182,6 @@ static void usage (void)
 	/*
 	 * Finish off the line.
 	 */
-
 	putchar ('\n');
 	exit (0);
 	/* NOT REACHED */
