@@ -29,15 +29,18 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: newgrp.c,v 1.32 2005/06/20 10:17:08 kloczek Exp $")
-#include <stdio.h>
+#ident "$Id: newgrp.c,v 1.41 2005/09/07 15:00:45 kloczek Exp $"
+
 #include <errno.h>
 #include <grp.h>
 #include <pwd.h>
-#include "prototypes.h"
+#include <stdio.h>
 #include "defines.h"
 #include "getdef.h"
+#include "prototypes.h"
+/*
+ * Global variables
+ */
 extern char **environ;
 
 #ifdef HAVE_SETGROUPS
@@ -54,7 +57,6 @@ static void usage (void);
 /*
  * usage - print command usage message
  */
-
 static void usage (void)
 {
 	if (is_newgrp)
@@ -63,10 +65,38 @@ static void usage (void)
 		fprintf (stderr, _("Usage: sg group [[-c] command]\n"));
 }
 
+/*
+ * find_matching_group - search all groups of a given group id for
+ *                       membership of a given username
+ */
+static struct group *find_matching_group (const char *name, gid_t gid)
+{
+	struct group *gr;
+	char **look;
+	int notfound = 1;
+
+	setgrent ();
+	while ((gr = getgrent ()) != NULL) {
+		if (gr->gr_gid != gid) {
+			continue;
+		}
+
+		/*
+		 * A group with matching GID was found.
+		 * Test for membership of 'name'.
+		 */
+		look = gr->gr_mem;
+		while (*look && (notfound = strcmp (*look++, name)));
+		if (!notfound)
+			break;
+	}
+	endgrent ();
+	return gr;
+}
+
 /*
  * newgrp - change the invokers current real and effective group id
  */
-
 int main (int argc, char **argv)
 {
 	int initflag = 0;
@@ -115,7 +145,6 @@ int main (int argc, char **argv)
 	 * but we do not need to restore the previous process persona and we
 	 * don't need to re-exec anything.  -- JWP
 	 */
-
 	Prog = Basename (argv[0]);
 	is_newgrp = (strcmp (Prog, "newgrp") == 0);
 	OPENLOG (is_newgrp ? "newgrp" : "sg");
@@ -151,7 +180,6 @@ int main (int argc, char **argv)
 	 *      sg [-]
 	 *      sg [-] groupid [[-c command]
 	 */
-
 	if (argc > 0 && (!strcmp (argv[0], "-") || !strcmp (argv[0], "-l"))) {
 		argc--;
 		argv++;
@@ -162,7 +190,6 @@ int main (int argc, char **argv)
 		 * Do the command line for everything that is
 		 * not "newgrp".
 		 */
-
 		if (argc > 0 && argv[0][0] != '-') {
 			group = argv[0];
 			argc--;
@@ -179,7 +206,6 @@ int main (int argc, char **argv)
 			 * "sg group -c command" (as in the man page) or
 			 * "sg group command" (as in the usage message).
 			 */
-
 			if (argc > 1 && strcmp (argv[0], "-c") == 0)
 				command = argv[1];
 			else
@@ -187,19 +213,16 @@ int main (int argc, char **argv)
 			cflag++;
 		}
 	} else {
-
 		/*
 		 * Do the command line for "newgrp". It's just making sure
 		 * there aren't any flags and getting the new group name.
 		 */
-
 		if (argc > 0 && argv[0][0] == '-') {
 			usage ();
 			goto failure;
 		} else if (argv[0] != (char *) 0) {
 			group = argv[0];
 		} else {
-
 			/*
 			 * get the group file entry for her login group id. 
 			 * the entry must exist, simply to be annoying.
@@ -207,7 +230,6 @@ int main (int argc, char **argv)
 			 * Perhaps in the past, but the default behavior now depends on the
 			 * group entry, so it had better exist.  -- JWP
 			 */
-
 			if (!(grp = getgrgid (pwd->pw_gid))) {
 				fprintf (stderr, _("unknown GID: %lu\n"),
 					 (unsigned long) pwd->pw_gid);
@@ -226,7 +248,6 @@ int main (int argc, char **argv)
 	 * nasty message but at least your real and effective group id's are
 	 * set.
 	 */
-
 	/* don't use getgroups(0, 0) - it doesn't work on some systems */
 	i = 16;
 	for (;;) {
@@ -284,11 +305,27 @@ int main (int argc, char **argv)
 	 * including the user's name in the member list of the user's login
 	 * group.  -- JWP
 	 */
-
 	if (!(grp = getgrnam (group))) {
 		fprintf (stderr, _("unknown group: %s\n"), group);
 		goto failure;
 	}
+
+	/*
+	 * For splitted groups (due to limitations of NIS), check all 
+	 * groups of the same GID like the requested group for
+	 * membership of the current user.
+	 */
+	grp = find_matching_group (name, grp->gr_gid);
+	if (!grp) {
+		/*
+		 * No matching group found. As we already know that
+		 * the group exists, this happens only in the case
+		 * of a requested group where the user is not member.
+		 *
+		 * Re-read the group entry for further processing.
+		 */
+		grp = getgrnam (group);
+	}
 #ifdef SHADOWGRP
 	if ((sgrp = getsgnam (group))) {
 		grp->gr_passwd = sgrp->sg_passwd;
@@ -302,7 +339,6 @@ int main (int argc, char **argv)
 	 * password, she will be denied access anyway.
 	 *
 	 */
-
 	if (!is_on_list (grp->gr_mem, name))
 		needspasswd = 1;
 
@@ -311,7 +347,6 @@ int main (int argc, char **argv)
 	 * password, and the group has a password, she needs to give the
 	 * group password.
 	 */
-
 	if ((spwd = getspnam (name)))
 		pwd->pw_passwd = spwd->sp_pwdp;
 
@@ -327,14 +362,11 @@ int main (int argc, char **argv)
 	 * Note that she now has to provide the password to her own group,
 	 * unless she is listed as a member.  -- JWP
 	 */
-
 	if (getuid () != 0 && needspasswd) {
-
 		/*
 		 * get the password from her, and set the salt for
 		 * the decryption from the group file.
 		 */
-
 		if (!(cp = getpass (_("Password: "))))
 			goto failure;
 
@@ -343,7 +375,6 @@ int main (int argc, char **argv)
 		 * password in the group file. The result of this encryption
 		 * must match the previously encrypted value in the file.
 		 */
-
 		cpasswd = pw_encrypt (cp, grp->gr_passwd);
 		strzero (cp);
 
@@ -370,7 +401,6 @@ int main (int argc, char **argv)
 	 * all successful validations pass through this point. The group id
 	 * will be set, and the group added to the concurrent groupset.
 	 */
-
 #ifdef	USE_SYSLOG
 	if (getdef_bool ("SYSLOG_SG_ENAB"))
 		SYSLOG ((LOG_INFO, "user `%s' switched to group `%s'",
@@ -394,58 +424,66 @@ int main (int argc, char **argv)
 			 "user `%s' (login `%s' on %s) switched to group `%s'",
 			 name, loginname, tty, group));
 #ifdef USE_PAM
-		if (getdef_bool ("CLOSE_SESSIONS")) {
-			/*
-			 * We want to fork and exec the new shell in the child, leaving the
-			 * parent waiting to log the session close.
-			 *
-			 * The parent must ignore signals generated from the console
-			 * (SIGINT, SIGQUIT, SIGHUP) which might make the parent terminate
-			 * before its child. When bash is exec'ed as the subshell, it
-			 * generates a new process group id for itself, and consequently
-			 * only SIGHUP, which is sent to all process groups in the session,
-			 * can reach the parent. However, since arbitrary programs can be
-			 * specified as login shells, there is no such guarantee in general.
-			 * For the same reason, we must also ignore stop signals generated
-			 * from the console (SIGTSTP, SIGTTIN, and SIGTTOU) in order to
-			 * avoid any possibility of the parent being stopped when it
-			 * receives SIGCHLD from the terminating subshell.  -- JWP
-			 */
-			pid_t child, pid;
+		/*
+		 * We want to fork and exec the new shell in the child, leaving the
+		 * parent waiting to log the session close.
+		 *
+		 * The parent must ignore signals generated from the console
+		 * (SIGINT, SIGQUIT, SIGHUP) which might make the parent terminate
+		 * before its child. When bash is exec'ed as the subshell, it
+		 * generates a new process group id for itself, and consequently
+		 * only SIGHUP, which is sent to all process groups in the session,
+		 * can reach the parent. However, since arbitrary programs can be
+		 * specified as login shells, there is no such guarantee in general.
+		 * For the same reason, we must also ignore stop signals generated
+		 * from the console (SIGTSTP, SIGTTIN, and SIGTTOU) in order to
+		 * avoid any possibility of the parent being stopped when it
+		 * receives SIGCHLD from the terminating subshell.  -- JWP
+		 */
+		pid_t child, pid;
 
-			signal (SIGINT, SIG_IGN);
-			signal (SIGQUIT, SIG_IGN);
-			signal (SIGHUP, SIG_IGN);
-			signal (SIGTSTP, SIG_IGN);
-			signal (SIGTTIN, SIG_IGN);
-			signal (SIGTTOU, SIG_IGN);
-			child = fork ();
-			if (child < 0) {
-				/* error in fork() */
-				fprintf (stderr, "%s: failure forking: %s",
-					 is_newgrp ? "newgrp" : "sg",
-					 strerror (errno));
-				exit (1);
-			} else if (child) {
-				/* parent - wait for child to finish, then log session close */
-				do {
-					pid = waitpid (child, NULL, 0);
-				} while (pid != child);
-				SYSLOG ((LOG_INFO,
-					 "user `%s' (login `%s' on %s) returned to group `%s'",
-					 name, loginname, tty,
-					 getgrgid (gid)->gr_name));
-				closelog ();
-				exit (0);
-			}
-			/* child - restore signals to their default state */
-			signal (SIGINT, SIG_DFL);
-			signal (SIGQUIT, SIG_DFL);
-			signal (SIGHUP, SIG_DFL);
-			signal (SIGTSTP, SIG_DFL);
-			signal (SIGTTIN, SIG_DFL);
-			signal (SIGTTOU, SIG_DFL);
+		signal (SIGINT, SIG_IGN);
+		signal (SIGQUIT, SIG_IGN);
+		signal (SIGHUP, SIG_IGN);
+		signal (SIGTSTP, SIG_IGN);
+		signal (SIGTTIN, SIG_IGN);
+		signal (SIGTTOU, SIG_IGN);
+		child = fork ();
+		if (child < 0) {
+			/* error in fork() */
+			fprintf (stderr, _("%s: failure forking: %s"),
+				 is_newgrp ? "newgrp" : "sg", strerror (errno));
+			exit (1);
+		} else if (child) {
+			/* parent - wait for child to finish, then log session close */
+			int cst = 0;
+
+			do {
+				errno = 0;
+				pid = waitpid (child, &cst, WUNTRACED);
+				if (pid == child && WIFSTOPPED (cst)) {
+					/* stop when child stops */
+					raise (SIGSTOP);
+					/* wake child when resumed */
+					kill (child, SIGCONT);
+				}
+			} while (pid == child && WIFSTOPPED (cst) ||
+				 pid != child && errno == EINTR);
+			SYSLOG ((LOG_INFO,
+				 "user `%s' (login `%s' on %s) returned to group `%s'",
+				 name, loginname, tty,
+				 getgrgid (gid)->gr_name));
+			closelog ();
+			exit (0);
 		}
+
+		/* child - restore signals to their default state */
+		signal (SIGINT, SIG_DFL);
+		signal (SIGQUIT, SIG_DFL);
+		signal (SIGHUP, SIG_DFL);
+		signal (SIGTSTP, SIG_DFL);
+		signal (SIGTTIN, SIG_DFL);
+		signal (SIGTTOU, SIG_DFL);
 #endif				/* USE_PAM */
 	}
 #endif				/* USE_SYSLOG */
@@ -459,7 +497,6 @@ int main (int argc, char **argv)
 	 * If the group doesn't fit, i'll complain loudly and skip this
 	 * part.
 	 */
-
 	for (i = 0; i < ngroups; i++) {
 		if (gid == grouplist[i])
 			break;
@@ -481,7 +518,6 @@ int main (int argc, char **argv)
 	 * to the real UID. For root, this also sets the real GID to the
 	 * new group id.
 	 */
-
 	if (setgid (gid))
 		perror ("setgid");
 
@@ -494,7 +530,6 @@ int main (int argc, char **argv)
 	 * See if the "-c" flag was used. If it was, i just create a shell
 	 * command for her using the argument that followed the "-c" flag.
 	 */
-
 	if (cflag) {
 		closelog ();
 		execl ("/bin/sh", "sh", "-c", command, (char *) 0);
@@ -524,7 +559,6 @@ int main (int argc, char **argv)
 	 * problem, try using $SHELL as a workaround; also please notify me
 	 * at jparmele@wildbear.com -- JWP
 	 */
-
 	if (!initflag && (cp = getenv ("SHELL")))
 		prog = cp;
 	else if (pwd->pw_shell && pwd->pw_shell[0])
@@ -536,7 +570,6 @@ int main (int argc, char **argv)
 	 * Now i try to find the basename of the login shell. This will
 	 * become argv[0] of the spawned command.
 	 */
-
 	cp = Basename ((char *) prog);
 
 	endspent ();
@@ -550,7 +583,6 @@ int main (int argc, char **argv)
 	 * Switch back to her home directory if i am doing login
 	 * initialization.
 	 */
-
 	if (initflag) {
 		if (chdir (pwd->pw_dir))
 			perror ("chdir");
@@ -573,7 +605,6 @@ int main (int argc, char **argv)
 	 * Exec the login shell and go away. We are trying to get back to
 	 * the previous environment which should be the user's login shell.
 	 */
-
 	shell (prog, initflag ? (char *) 0 : cp);
 	/* NOTREACHED */
       failure:
@@ -588,7 +619,6 @@ int main (int argc, char **argv)
 	 * process. The closelog is probably unnecessary, but it does no
 	 * harm.  -- JWP
 	 */
-
 	closelog ();
 	exit (1);
 }
