@@ -29,34 +29,35 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: usermod.c,v 1.31 2003/06/30 13:17:51 kloczek Exp $")
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdio.h>
-#include <errno.h>
-#include <pwd.h>
-#include <grp.h>
+#ident "$Id: usermod.c,v 1.58 2005/10/04 21:05:12 kloczek Exp $"
+
 #include <ctype.h>
+#include <errno.h>
 #include <fcntl.h>
-#include <time.h>
+#include <grp.h>
+#include <lastlog.h>
+#include <pwd.h>
 #ifdef USE_PAM
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
-#include <pwd.h>
 #endif				/* USE_PAM */
-#include "prototypes.h"
-#include "defines.h"
+#include <stdio.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
 #include "chkname.h"
+#include "defines.h"
 #include "faillog.h"
-#if HAVE_LASTLOG_H
-#include <lastlog.h>
-#else
-#include "lastlog_.h"
-#endif
-#include "pwauth.h"
-#include "nscd.h"
 #include "getdef.h"
+#include "groupio.h"
+#include "nscd.h"
+#include "prototypes.h"
+#include "pwauth.h"
+#include "pwio.h"
+#ifdef	SHADOWGRP
+#include "sgroupio.h"
+#endif
+#include "shadowio.h"
 /*
  * exit status values
  * for E_GRP_UPDATE and E_NOSPACE (not used yet), other update requests
@@ -66,15 +67,18 @@ RCSID (PKG_VER "$Id: usermod.c,v 1.31 2003/06/30 13:17:51 kloczek Exp $")
 #define E_PW_UPDATE	1	/* can't update password file */
 #define E_USAGE		2	/* invalid command syntax */
 #define E_BAD_ARG	3	/* invalid argument to option */
-#define E_UID_IN_USE	4	/* uid already in use (and no -o) */
-    /* #define E_BAD_PWFILE	5 *//* passwd file contains errors */
+#define E_UID_IN_USE	4	/* UID already in use (and no -o) */
+				/* #define E_BAD_PWFILE     5 *//* passwd file contains errors */
 #define E_NOTFOUND	6	/* specified user/group doesn't exist */
 #define E_USER_BUSY	8	/* user to modify is logged in */
 #define E_NAME_IN_USE	9	/* username already in use */
 #define E_GRP_UPDATE	10	/* can't update group file */
-    /* #define E_NOSPACE	11 *//* insufficient space to move home dir */
+				 /* #define E_NOSPACE        11 *//* insufficient space to move home dir */
 #define E_HOMEDIR	12	/* unable to complete home dir move */
 #define	VALID(s)	(strcspn (s, ":\n") == strlen (s))
+/*
+ * Global variables
+ */
 static char *user_name;
 static char *user_newname;
 static char *user_pass;
@@ -83,77 +87,51 @@ static uid_t user_newid;
 static gid_t user_gid;
 static gid_t user_newgid;
 static char *user_comment;
+static char *user_newcomment;	/* Audit */
 static char *user_home;
 static char *user_newhome;
 static char *user_shell;
+static char *user_newshell;	/* Audit */
 
-#ifdef	SHADOWPWD
 static long user_expire;
+static long user_newexpire;	/* Audit */
 static long user_inactive;
-#endif
+static long user_newinactive;	/* Audit */
 static long sys_ngroups;
 static char **user_groups;	/* NULL-terminated list */
 
 static char *Prog;
 
 static int
- uflg = 0,			/* specify new user ID */
- oflg = 0,			/* permit non-unique user ID to be specified with -u */
- gflg = 0,			/* new primary group ID */
- Gflg = 0,			/* new secondary group set */
- dflg = 0,			/* new home directory */
- sflg = 0,			/* new shell program */
- cflg = 0,			/* new comment (GECOS) field */
- mflg = 0,			/* create user's home directory if it doesn't exist */
-#ifdef SHADOWPWD
- fflg = 0,			/* days until account with expired password is locked */
- eflg = 0,			/* days since 1970-01-01 when account becomes expired */
-#endif
- Lflg = 0,			/* lock the password */
- Uflg = 0,			/* unlock the password */
- pflg = 0,			/* new encrypted password */
- lflg = 0;			/* new user name */
+ aflg = 0,			/* append to existing secondary group set */
+    cflg = 0,			/* new comment (GECOS) field */
+    dflg = 0,			/* new home directory */
+    eflg = 0,			/* days since 1970-01-01 when account becomes expired */
+    fflg = 0,			/* days until account with expired password is locked */
+    gflg = 0,			/* new primary group ID */
+    Gflg = 0,			/* new secondary group set */
+    Lflg = 0,			/* lock the password */
+    lflg = 0,			/* new user name */
+    mflg = 0,			/* create user's home directory if it doesn't exist */
+    oflg = 0,			/* permit non-unique user ID to be specified with -u */
+    pflg = 0,			/* new encrypted password */
+    sflg = 0,			/* new shell program */
+    uflg = 0,			/* specify new user ID */
+    Uflg = 0;			/* unlock the password */
 
-#ifdef	NDBM
-extern int pw_dbm_mode;
-
-#ifdef	SHADOWPWD
-extern int sp_dbm_mode;
-#endif
-extern int gr_dbm_mode;
-
-#ifdef	SHADOWGRP
-extern int sg_dbm_mode;
-#endif
-#endif
-
-#ifdef SHADOWPWD
 static int is_shadow_pwd;
-#endif
+
 #ifdef SHADOWGRP
 static int is_shadow_grp;
 #endif
 
-#include "groupio.h"
-
-#ifdef	SHADOWGRP
-#include "sgroupio.h"
-#endif
-
-#include "pwio.h"
-
-#ifdef	SHADOWPWD
-#include "shadowio.h"
-#endif
 
 /* local function prototypes */
 static int get_groups (char *);
 static void usage (void);
 static void new_pwent (struct passwd *);
 
-#ifdef SHADOWPWD
 static void new_spent (struct spwd *);
-#endif
 static void fail_exit (int);
 static int update_group (void);
 
@@ -186,13 +164,12 @@ static struct group *getgr_nam_gid (const char *name)
 	char *ep;
 
 	gid = strtoul (name, &ep, 10);
-	if (*name != '\0' && *ep == '\0')	/* valid numeric gid */
+	if (*name != '\0' && *ep == '\0')	/* valid numeric GID */
 		return getgrgid (gid);
 
 	return getgrnam (name);
 }
 
-
 /*
  * get_groups - convert a list of group names to an array of group IDs
  *
@@ -200,7 +177,6 @@ static struct group *getgr_nam_gid (const char *name)
  *	converts it to a NULL-terminated array. Any unknown group names are
  *	reported as errors.
  */
-
 static int get_groups (char *list)
 {
 	char *cp;
@@ -211,7 +187,6 @@ static int get_groups (char *list)
 	/*
 	 * Initialize the list to be empty
 	 */
-
 	user_groups[0] = (char *) 0;
 
 	if (!*list)
@@ -222,7 +197,6 @@ static int get_groups (char *list)
 	 * name and look it up. A mix of numerical and string values for
 	 * group identifiers is permitted.
 	 */
-
 	do {
 		/*
 		 * Strip off a single name from the list
@@ -259,7 +233,6 @@ static int get_groups (char *list)
 		 * Don't add this group if they are an NIS group. Tell the
 		 * user to go to the server for this group.
 		 */
-
 		if (__isgrNIS ()) {
 			fprintf (stderr,
 				 _("%s: group `%s' is a NIS group.\n"),
@@ -279,7 +252,6 @@ static int get_groups (char *list)
 		/*
 		 * Add the group name to the user's list of groups.
 		 */
-
 		user_groups[ngroups++] = xstrdup (grp->gr_name);
 	} while (list);
 
@@ -288,7 +260,6 @@ static int get_groups (char *list)
 	/*
 	 * Any errors in finding group names are fatal
 	 */
-
 	if (errors)
 		return -1;
 
@@ -298,20 +269,17 @@ static int get_groups (char *list)
 /*
  * usage - display usage message and exit
  */
-
 static void usage (void)
 {
 	fprintf (stderr,
 		 _
-		 ("Usage: %s\t[-u uid [-o]] [-g group] [-G group,...] \n"),
+		 ("Usage: %s\t[-u uid [-o]] [-g group] [[-G group,...] [-a]] \n"),
 		 Prog);
 	fprintf (stderr,
 		 _
 		 ("\t\t[-d home [-m]] [-s shell] [-c comment] [-l new_name]\n"));
 	fprintf (stderr, "\t\t");
-#ifdef SHADOWPWD
 	fprintf (stderr, _("[-f inactive] [-e expire] "));
-#endif
 	fprintf (stderr, _("[-p passwd] [-L|-U] name\n"));
 	exit (E_USAGE);
 }
@@ -320,12 +288,15 @@ static void usage (void)
  * update encrypted password string (for both shadow and non-shadow
  * passwords)
  */
-
 static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 {
 	if (Lflg && pw_pass[0] != '!') {
 		char *buf = xmalloc (strlen (pw_pass) + 2);
 
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "updating passwd",
+			      user_newname, user_newid, 0);
+#endif
 		SYSLOG ((LOG_INFO, "lock user `%s' password", pw_name));
 		strcpy (buf, "!");
 		strcat (buf, pw_pass);
@@ -333,6 +304,10 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 	} else if (Uflg && pw_pass[0] == '!') {
 		char *s;
 
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "updating password",
+			      user_newname, user_newid, 0);
+#endif
 		SYSLOG ((LOG_INFO, "unlock user `%s' password", pw_name));
 		s = pw_pass;
 		while (*s) {
@@ -340,6 +315,10 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 			s++;
 		}
 	} else if (pflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing password",
+			      user_newname, user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO, "change user `%s' password", pw_name));
 		pw_pass = xstrdup (user_pass);
 	}
@@ -352,83 +331,172 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
  *	new_pwent() takes all of the values that have been entered and fills
  *	in a (struct passwd) with them.
  */
-
 static void new_pwent (struct passwd *pwent)
 {
 	if (lflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing name",
+			      user_newname, user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO, "change user name `%s' to `%s'",
 			 pwent->pw_name, user_newname));
 		pwent->pw_name = xstrdup (user_newname);
 	}
-#ifdef SHADOWPWD
 	if (!is_shadow_pwd)
-#endif
 		pwent->pw_passwd =
 		    new_pw_passwd (pwent->pw_passwd, pwent->pw_name);
 
 	if (uflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing uid",
+			      user_newname, user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' UID from `%d' to `%d'",
 			 pwent->pw_name, pwent->pw_uid, user_newid));
 		pwent->pw_uid = user_newid;
 	}
 	if (gflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "changing primary group", user_newname,
+			      user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' GID from `%d' to `%d'",
 			 pwent->pw_name, pwent->pw_gid, user_newgid));
 		pwent->pw_gid = user_newgid;
 	}
-	if (cflg)
+	if (cflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing comment",
+			      user_newname, user_newid, 1);
+		pwent->pw_gecos = user_newcomment;
+#else
 		pwent->pw_gecos = user_comment;
+#endif
+	}
 
 	if (dflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "changing home directory", user_newname,
+			      user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' home from `%s' to `%s'",
 			 pwent->pw_name, pwent->pw_dir, user_newhome));
 		pwent->pw_dir = user_newhome;
 	}
 	if (sflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing user shell",
+			      user_newname, user_newid, 1);
+		SYSLOG ((LOG_INFO, "change user `%s' shell from `%s' to `%s'",
+			 pwent->pw_name, pwent->pw_shell, user_newshell));
+		pwent->pw_shell = user_newshell;
+#else
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' shell from `%s' to `%s'",
 			 pwent->pw_name, pwent->pw_shell, user_shell));
 		pwent->pw_shell = user_shell;
+#endif
 	}
 }
 
-#ifdef	SHADOWPWD
 /*
  * new_spent - initialize the values in a shadow password file entry
  *
  *	new_spent() takes all of the values that have been entered and fills
  *	in a (struct spwd) with them.
  */
-
 static void new_spent (struct spwd *spent)
 {
 	if (lflg)
 		spent->sp_namp = xstrdup (user_newname);
 
 	if (fflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "changing inactive days", user_newname,
+			      user_newid, 1);
+		SYSLOG ((LOG_INFO,
+			 "change user `%s' inactive from `%ld' to `%ld'",
+			 spent->sp_namp, spent->sp_inact, user_newinactive));
+		spent->sp_inact = user_newinactive;
+#else
+
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' inactive from `%ld' to `%ld'",
 			 spent->sp_namp, spent->sp_inact, user_inactive));
 		spent->sp_inact = user_inactive;
+#endif
 	}
 	if (eflg) {
 		/* XXX - dates might be better than numbers of days.  --marekm */
+#ifdef WITH_AUDIT
+		if (audit_fd >= 0) {
+			time_t exp_t;
+			struct tm *exp_tm;
+			char new_exp[16], old_exp[16];
+
+			if (user_newexpire == -1)
+				new_exp[0] = '\0';
+			else {
+				exp_t = user_newexpire * DAY;
+				exp_tm = gmtime (&exp_t);
+#ifdef HAVE_STRFTIME
+				strftime (new_exp, sizeof (new_exp), "%Y-%m-%d",
+					  exp_tm);
+#else
+				memset (new_exp, 0, sizeof (new_exp));
+				snprintf (new_exp, sizeof (new_exp) - 1,
+					  "%04i-%02i-%02i",
+					  exp_tm->tm_year + 1900,
+					  exp_tm->tm_mon + 1, exp_tm->tm_mday);
+#endif
+			}
+
+			if (user_expire == -1)
+				old_exp[0] = '\0';
+			else {
+				exp_t = user_expire * DAY;
+				exp_tm = gmtime (&exp_t);
+#ifdef HAVE_STRFTIME
+				strftime (old_exp, sizeof (old_exp), "%Y-%m-%d",
+					  exp_tm);
+#else
+				memset (old_exp, 0, sizeof (old_exp));
+				snprintf (old_exp, sizeof (old_exp) - 1,
+					  "%04i-%02i-%02i",
+					  exp_tm->tm_year + 1900,
+					  exp_tm->tm_mon + 1, exp_tm->tm_mday);
+#endif
+			}
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "changing expiration date", user_newname,
+				      user_newid, 1);
+		}
+
+		SYSLOG ((LOG_INFO,
+			 "change user `%s' expiration from `%ld' to `%ld'",
+			 spent->sp_namp, spent->sp_expire, user_newexpire));
+		spent->sp_expire = user_newexpire;
+#else
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' expiration from `%ld' to `%ld'",
 			 spent->sp_namp, spent->sp_expire, user_expire));
 		spent->sp_expire = user_expire;
+#endif
 	}
 	spent->sp_pwdp = new_pw_passwd (spent->sp_pwdp, spent->sp_namp);
+	if (pflg)
+		spent->sp_lstchg = time ((time_t *) 0) / SCALE;
 }
-#endif				/* SHADOWPWD */
 
 /*
  * fail_exit - exit with an error code after unlocking files
  */
-
 static void fail_exit (int code)
 {
 	(void) gr_unlock ();
@@ -436,11 +504,13 @@ static void fail_exit (int code)
 	if (is_shadow_grp)
 		sgr_unlock ();
 #endif
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd)
 		spw_unlock ();
-#endif
 	(void) pw_unlock ();
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying account",
+		      user_name, -1, 0);
+#endif
 	exit (code);
 }
 
@@ -458,14 +528,12 @@ static int update_group (void)
 	 * entries.
 	 */
 	if (!gr_lock ()) {
-		fprintf (stderr, _("%s: error locking group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error locking group file\n"), Prog);
 		SYSLOG ((LOG_ERR, "error locking group file"));
 		return -1;
 	}
 	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: error opening group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error opening group file\n"), Prog);
 		SYSLOG ((LOG_ERR, "error opening group file"));
 		gr_unlock ();
 		return -1;
@@ -478,7 +546,6 @@ static int update_group (void)
 	 * the user is a member of.
 	 */
 	while ((grp = gr_next ())) {
-
 		/*
 		 * See if the user specified this group as one of their
 		 * concurrent groups.
@@ -505,14 +572,24 @@ static int update_group (void)
 				ngrp->gr_mem = add_list (ngrp->gr_mem,
 							 user_newname);
 				changed = 1;
+#ifdef WITH_AUDIT
+				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+					      "changing group member",
+					      user_newname, -1, 1);
+#endif
 				SYSLOG ((LOG_INFO,
 					 "change `%s' to `%s' in group `%s'",
 					 user_name, user_newname,
 					 ngrp->gr_name));
 			}
-		} else if (was_member && Gflg && !is_member) {
+		} else if (was_member && !aflg && Gflg && !is_member) {
 			ngrp->gr_mem = del_list (ngrp->gr_mem, user_name);
 			changed = 1;
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "removing group member", user_name, -1,
+				      1);
+#endif
 			SYSLOG ((LOG_INFO, "delete `%s' from group `%s'",
 				 user_name, ngrp->gr_name));
 		} else if (!was_member && Gflg && is_member) {
@@ -520,6 +597,10 @@ static int update_group (void)
 						 lflg ? user_newname :
 						 user_name);
 			changed = 1;
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "adding user to group", user_name, -1, 1);
+#endif
 			SYSLOG ((LOG_INFO, "add `%s' to group `%s'",
 				 lflg ? user_newname : user_name,
 				 ngrp->gr_name));
@@ -530,32 +611,15 @@ static int update_group (void)
 		changed = 0;
 		if (!gr_update (ngrp)) {
 			fprintf (stderr,
-				 _("%s: error adding new group entry\n"),
-				 Prog);
+				 _("%s: error adding new group entry\n"), Prog);
 			SYSLOG ((LOG_ERR, "error adding group entry"));
 			gr_unlock ();
 			return -1;
 		}
-#ifdef	NDBM
-		/*
-		 * Update the DBM group file with the new entry as well.
-		 */
-		if (!gr_dbm_update (ngrp)) {
-			fprintf (stderr,
-				 _("%s: cannot add new dbm group entry\n"),
-				 Prog);
-			SYSLOG ((LOG_ERR, "error adding dbm group entry"));
-			gr_unlock ();
-			return -1;
-		}
-#endif				/* NDBM */
 	}
-#ifdef NDBM
-	endgrent ();
-#endif				/* NDBM */
+
 	if (!gr_close ()) {
-		fprintf (stderr, _("%s: cannot rewrite group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: cannot rewrite group file\n"), Prog);
 		gr_unlock ();
 		return -1;
 	}
@@ -609,8 +673,7 @@ static int update_gshadow (void)
 		 * See if the user specified this group as one of their
 		 * concurrent groups.
 		 */
-		is_member = Gflg
-		    && is_on_list (user_groups, sgrp->sg_name);
+		is_member = Gflg && is_on_list (user_groups, sgrp->sg_name);
 
 		if (!was_member && !was_admin && !is_member)
 			continue;
@@ -626,11 +689,14 @@ static int update_gshadow (void)
 		}
 
 		if (was_admin && lflg) {
-			nsgrp->sg_adm =
-			    del_list (nsgrp->sg_adm, user_name);
-			nsgrp->sg_adm =
-			    add_list (nsgrp->sg_adm, user_newname);
+			nsgrp->sg_adm = del_list (nsgrp->sg_adm, user_name);
+			nsgrp->sg_adm = add_list (nsgrp->sg_adm, user_newname);
 			changed = 1;
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "changing admin name in shadow group",
+				      user_name, -1, 1);
+#endif
 			SYSLOG ((LOG_INFO,
 				 "change admin `%s' to `%s' in shadow group `%s'",
 				 user_name, user_newname, nsgrp->sg_name));
@@ -642,15 +708,24 @@ static int update_gshadow (void)
 				nsgrp->sg_mem = add_list (nsgrp->sg_mem,
 							  user_newname);
 				changed = 1;
+#ifdef WITH_AUDIT
+				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+					      "changing member in shadow group",
+					      user_name, -1, 1);
+#endif
 				SYSLOG ((LOG_INFO,
 					 "change `%s' to `%s' in shadow group `%s'",
 					 user_name, user_newname,
 					 nsgrp->sg_name));
 			}
-		} else if (was_member && Gflg && !is_member) {
-			nsgrp->sg_mem =
-			    del_list (nsgrp->sg_mem, user_name);
+		} else if (was_member && !aflg && Gflg && !is_member) {
+			nsgrp->sg_mem = del_list (nsgrp->sg_mem, user_name);
 			changed = 1;
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "removing user from shadow group",
+				      user_name, -1, 1);
+#endif
 			SYSLOG ((LOG_INFO,
 				 "delete `%s' from shadow group `%s'",
 				 user_name, nsgrp->sg_name));
@@ -659,6 +734,11 @@ static int update_gshadow (void)
 						  lflg ? user_newname :
 						  user_name);
 			changed = 1;
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "adding user to shadow group",
+				      user_newname, -1, 1);
+#endif
 			SYSLOG ((LOG_INFO, "add `%s' to shadow group `%s'",
 				 lflg ? user_newname : user_name,
 				 nsgrp->sg_name));
@@ -673,35 +753,16 @@ static int update_gshadow (void)
 		 */
 		if (!sgr_update (nsgrp)) {
 			fprintf (stderr,
-				 _("%s: error adding new group entry\n"),
-				 Prog);
-			SYSLOG ((LOG_ERR,
-				 "error adding shadow group entry"));
+				 _("%s: error adding new group entry\n"), Prog);
+			SYSLOG ((LOG_ERR, "error adding shadow group entry"));
 			sgr_unlock ();
 			return -1;
 		}
-#ifdef	NDBM
-		/*
-		 * Update the DBM group file with the new entry as well.
-		 */
-		if (!sg_dbm_update (nsgrp)) {
-			fprintf (stderr,
-				 _("%s: cannot add new dbm group entry\n"),
-				 Prog);
-			SYSLOG ((LOG_ERR,
-				 "error adding dbm shadow group entry"));
-			sgr_unlock ();
-			return -1;
-		}
-#endif				/* NDBM */
 	}
-#ifdef NDBM
-	endsgent ();
-#endif				/* NDBM */
+
 	if (!sgr_close ()) {
 		fprintf (stderr,
-			 _("%s: cannot rewrite shadow group file\n"),
-			 Prog);
+			 _("%s: cannot rewrite shadow group file\n"), Prog);
 		sgr_unlock ();
 		return -1;
 	}
@@ -716,7 +777,6 @@ static int update_gshadow (void)
  *	grp_update() takes the secondary group set given in user_groups and
  *	adds the user to each group given by that set.
  */
-
 static int grp_update (void)
 {
 	int ret;
@@ -738,8 +798,7 @@ static long get_number (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog,
-		 cp);
+	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -752,8 +811,7 @@ static uid_t get_id (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog,
-		 cp);
+	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -764,15 +822,12 @@ static uid_t get_id (const char *cp)
  *	values that the user will be created with accordingly. The values
  *	are checked for sanity.
  */
-
 static void process_flags (int argc, char **argv)
 {
 	const struct group *grp;
 	const struct passwd *pwd;
 
-#ifdef SHADOWPWD
 	const struct spwd *spwd = NULL;
-#endif
 	int anyflag = 0;
 	int arg;
 
@@ -784,14 +839,26 @@ static void process_flags (int argc, char **argv)
 			 Prog, argv[argc - 1]);
 		exit (E_NOTFOUND);
 	}
+
 	user_name = argv[argc - 1];
+	user_id = pwd->pw_uid;
+	user_gid = pwd->pw_gid;
+	user_comment = xstrdup (pwd->pw_gecos);
+	user_home = xstrdup (pwd->pw_dir);
+	user_shell = xstrdup (pwd->pw_shell);
+#ifdef WITH_AUDIT
+	user_newname = user_name;
+	user_newid = user_id;
+	user_newgid = user_gid;
+	user_newcomment = user_comment;
+	user_newhome = user_home;
+	user_newshell = user_shell;
+#endif
 
 #ifdef	USE_NIS
-
 	/*
 	 * Now make sure it isn't an NIS user.
 	 */
-
 	if (__ispwNIS ()) {
 		char *nis_domain;
 		char *nis_master;
@@ -800,34 +867,28 @@ static void process_flags (int argc, char **argv)
 			 Prog, user_name);
 
 		if (!yp_get_default_domain (&nis_domain) &&
-		    !yp_master (nis_domain, "passwd.byname",
-				&nis_master)) {
+		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
 			fprintf (stderr, _("%s: %s is the NIS master\n"),
 				 Prog, nis_master);
 		}
 		exit (E_NOTFOUND);
 	}
 #endif
-	user_id = pwd->pw_uid;
-	user_gid = pwd->pw_gid;
-	user_comment = xstrdup (pwd->pw_gecos);
-	user_home = xstrdup (pwd->pw_dir);
-	user_shell = xstrdup (pwd->pw_shell);
 
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd && (spwd = getspnam (user_name))) {
 		user_expire = spwd->sp_expire;
 		user_inactive = spwd->sp_inact;
+#ifdef WITH_AUDIT
+		user_newexpire = user_expire;
+		user_newinactive = user_inactive;
+#endif
 	}
-#endif
-#ifdef	SHADOWPWD
-#define FLAGS "A:u:og:G:d:s:c:mf:e:l:p:LU"
-#else
-#define FLAGS "A:u:og:G:d:s:c:ml:p:LU"
-#endif
-	while ((arg = getopt (argc, argv, FLAGS)) != EOF) {
-#undef FLAGS
+
+	while ((arg = getopt (argc, argv, "ac:d:e:f:g:G:l:Lmop:s:u:U")) != EOF) {
 		switch (arg) {
+		case 'a':
+			aflg++;
+			break;
 		case 'c':
 			if (!VALID (optarg)) {
 				fprintf (stderr,
@@ -835,7 +896,11 @@ static void process_flags (int argc, char **argv)
 					 Prog, optarg);
 				exit (E_BAD_ARG);
 			}
+#ifdef WITH_AUDIT
+			user_newcomment = optarg;
+#else
 			user_comment = optarg;
+#endif
 			cflg++;
 			break;
 		case 'd':
@@ -848,27 +913,42 @@ static void process_flags (int argc, char **argv)
 			dflg++;
 			user_newhome = optarg;
 			break;
-#ifdef SHADOWPWD
 		case 'e':
 			if (*optarg) {
+#ifdef WITH_AUDIT
+				user_newexpire = strtoday (optarg);
+				if (user_newexpire == -1) {
+#else
 				user_expire = strtoday (optarg);
 				if (user_expire == -1) {
+#endif
 					fprintf (stderr,
 						 _
 						 ("%s: invalid date `%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
+#ifdef WITH_AUDIT
+				user_newexpire *= DAY / SCALE;
+#else
 				user_expire *= DAY / SCALE;
+#endif
 			} else
+#ifdef WITH_AUDIT
+				user_newexpire = -1;
+#else
 				user_expire = -1;
+#endif
 			eflg++;
 			break;
 		case 'f':
+#ifdef WITH_AUDIT
+			user_newinactive = get_number (optarg);
+#else
 			user_inactive = get_number (optarg);
+#endif
 			fflg++;
 			break;
-#endif
 		case 'g':
 			grp = getgr_nam_gid (optarg);
 			if (!grp) {
@@ -898,7 +978,6 @@ static void process_flags (int argc, char **argv)
 			 * set the flag as this will cause rather serious
 			 * problems later!
 			 */
-
 			if (strcmp (user_name, optarg))
 				lflg++;
 
@@ -936,7 +1015,11 @@ static void process_flags (int argc, char **argv)
 					 Prog, optarg);
 				exit (E_BAD_ARG);
 			}
+#ifdef WITH_AUDIT
+			user_newshell = optarg;
+#else
 			user_shell = optarg;
+#endif
 			sflg++;
 			break;
 		case 'u':
@@ -958,7 +1041,6 @@ static void process_flags (int argc, char **argv)
 		fprintf (stderr, _("%s: no flags given\n"), Prog);
 		exit (E_USAGE);
 	}
-#ifdef SHADOWPWD
 	if (!is_shadow_pwd && (eflg || fflg)) {
 		fprintf (stderr,
 			 _
@@ -966,11 +1048,18 @@ static void process_flags (int argc, char **argv)
 			 Prog);
 		exit (E_USAGE);
 	}
-#endif
 
 	if (optind != argc - 1)
 		usage ();
 
+	if (aflg && (!Gflg)) {
+		fprintf (stderr,
+			 _("%s: -a flag is ONLY allowed with the -G flag\n"),
+			 Prog);
+		usage ();
+		exit (E_USAGE);
+	}
+
 	if (dflg && strcmp (user_home, user_newhome) == 0)
 		dflg = mflg = 0;
 
@@ -978,8 +1067,7 @@ static void process_flags (int argc, char **argv)
 		uflg = oflg = 0;
 
 	if (lflg && getpwnam (user_newname)) {
-		fprintf (stderr, _("%s: user %s exists\n"), Prog,
-			 user_newname);
+		fprintf (stderr, _("%s: user %s exists\n"), Prog, user_newname);
 		exit (E_NAME_IN_USE);
 	}
 
@@ -996,36 +1084,26 @@ static void process_flags (int argc, char **argv)
  *	close_files() closes all of the files that were opened for this new
  *	user. This causes any modified entries to be written out.
  */
-
 static void close_files (void)
 {
 	if (!pw_close ()) {
-		fprintf (stderr, _("%s: cannot rewrite password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: cannot rewrite password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd && !spw_close ()) {
 		fprintf (stderr,
-			 _("%s: cannot rewrite shadow password file\n"),
-			 Prog);
+			 _("%s: cannot rewrite shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
-#endif
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd)
 		spw_unlock ();
-#endif
 	(void) pw_unlock ();
 
 	/*
 	 * Close the DBM and/or flat files
 	 */
-
 	endpwent ();
-#ifdef	SHADOWPWD
 	endspent ();
-#endif
 	endgrent ();
 #ifdef	SHADOWGRP
 	endsgent ();
@@ -1037,33 +1115,26 @@ static void close_files (void)
  *
  *	open_files() opens the two password files.
  */
-
 static void open_files (void)
 {
 	if (!pw_lock ()) {
-		fprintf (stderr, _("%s: unable to lock password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: unable to lock password file\n"), Prog);
 		exit (E_PW_UPDATE);
 	}
 	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("%s: unable to open password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: unable to open password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd && !spw_lock ()) {
 		fprintf (stderr,
-			 _("%s: cannot lock shadow password file\n"),
-			 Prog);
+			 _("%s: cannot lock shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
 	if (is_shadow_pwd && !spw_open (O_RDWR)) {
 		fprintf (stderr,
-			 _("%s: cannot open shadow password file\n"),
-			 Prog);
+			 _("%s: cannot open shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
-#endif
 }
 
 /*
@@ -1072,21 +1143,17 @@ static void open_files (void)
  *	usr_update() creates the password file entries for this user and
  *	will update the group entries if required.
  */
-
 static void usr_update (void)
 {
 	struct passwd pwent;
 	const struct passwd *pwd;
 
-#ifdef	SHADOWPWD
 	struct spwd spent;
 	const struct spwd *spwd = NULL;
-#endif
 
 	/*
 	 * Locate the entry in /etc/passwd, which MUST exist.
 	 */
-
 	pwd = pw_locate (user_name);
 	if (!pwd) {
 		fprintf (stderr, _("%s: %s not found in /etc/passwd\n"),
@@ -1096,18 +1163,15 @@ static void usr_update (void)
 	pwent = *pwd;
 	new_pwent (&pwent);
 
-#ifdef	SHADOWPWD
 
 	/* 
 	 * Locate the entry in /etc/shadow. It doesn't have to exist, and
 	 * won't be created if it doesn't.
 	 */
-
 	if (is_shadow_pwd && (spwd = spw_locate (user_name))) {
 		spent = *spwd;
 		new_spent (&spent);
 	}
-#endif
 
 	if (lflg || uflg || gflg || cflg || dflg || sflg || pflg
 	    || Lflg || Uflg) {
@@ -1123,27 +1187,7 @@ static void usr_update (void)
 				 Prog);
 			fail_exit (E_PW_UPDATE);
 		}
-#ifdef NDBM
-		if (pw_dbm_present ()) {
-			if (!pw_dbm_update (&pwent)) {
-				fprintf (stderr,
-					 _
-					 ("%s: error adding password dbm entry\n"),
-					 Prog);
-				fail_exit (E_PW_UPDATE);
-			}
-			if (lflg && (pwd = getpwnam (user_name)) &&
-			    !pw_dbm_remove (pwd)) {
-				fprintf (stderr,
-					 _
-					 ("%s: error removing passwd dbm entry\n"),
-					 Prog);
-				fail_exit (E_PW_UPDATE);
-			}
-		}
-#endif
 	}
-#ifdef	SHADOWPWD
 	if (spwd && (lflg || eflg || fflg || pflg || Lflg || Uflg)) {
 		if (!spw_update (&spent)) {
 			fprintf (stderr,
@@ -1160,25 +1204,6 @@ static void usr_update (void)
 			fail_exit (E_PW_UPDATE);
 		}
 	}
-#ifdef	NDBM
-	if (spwd && sp_dbm_present ()) {
-		if (!sp_dbm_update (&spent)) {
-			fprintf (stderr,
-				 _
-				 ("%s: error updating shadow passwd dbm entry\n"),
-				 Prog);
-			fail_exit (E_PW_UPDATE);
-		}
-		if (lflg && !sp_dbm_remove (user_name)) {
-			fprintf (stderr,
-				 _
-				 ("%s: error removing shadow passwd dbm entry\n"),
-				 Prog);
-			fail_exit (E_PW_UPDATE);
-		}
-	}
-#endif				/* NDBM */
-#endif				/* SHADOWPWD */
 }
 
 /*
@@ -1187,7 +1212,6 @@ static void usr_update (void)
  *	move_home() moves the user's home directory to a new location. The
  *	files will be copied if the directory cannot simply be renamed.
  */
-
 static void move_home (void)
 {
 	struct stat sb;
@@ -1206,15 +1230,13 @@ static void move_home (void)
 			fail_exit (E_HOMEDIR);
 		} else if (rename (user_home, user_newhome)) {
 			if (errno == EXDEV) {
-				if (mkdir
-				    (user_newhome, sb.st_mode & 0777)) {
+				if (mkdir (user_newhome, sb.st_mode & 0777)) {
 					fprintf (stderr,
 						 _
 						 ("%s: can't create %s\n"),
 						 Prog, user_newhome);
 				}
-				if (chown (user_newhome,
-					   sb.st_uid, sb.st_gid)) {
+				if (chown (user_newhome, sb.st_uid, sb.st_gid)) {
 					fprintf (stderr,
 						 _("%s: can't chown %s\n"),
 						 Prog, user_newhome);
@@ -1223,10 +1245,22 @@ static void move_home (void)
 				}
 				if (copy_tree (user_home, user_newhome,
 					       uflg ? user_newid : -1,
-					       gflg ? user_newgid : -1) ==
-				    0 && remove_tree (user_home) == 0
-				    && rmdir (user_home) == 0)
+					       gflg ? user_newgid : -1) == 0) {
+					if (remove_tree (user_home) != 0 ||
+					    rmdir (user_home) != 0)
+						fprintf (stderr,
+							 _
+							 ("%s: warning: failed to completely remove old home directory %s"),
+							 Prog, user_home);
+#ifdef WITH_AUDIT
+					audit_logger (AUDIT_USER_CHAUTHTOK,
+						      Prog,
+						      "moving home directory",
+						      user_newname, user_newid,
+						      1);
+#endif
 					return;
+				}
 
 				(void) remove_tree (user_newhome);
 				(void) rmdir (user_newhome);
@@ -1237,17 +1271,27 @@ static void move_home (void)
 				 Prog, user_home, user_newhome);
 			fail_exit (E_HOMEDIR);
 		}
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "moving home directory", user_newname, user_newid,
+			      1);
+#endif
 	}
-	if (uflg || gflg)
+	if (uflg || gflg) {
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "changing home directory owner", user_newname,
+			      user_newid, 1);
+#endif
 		chown (dflg ? user_newhome : user_home,
 		       uflg ? user_newid : user_id,
 		       gflg ? user_newgid : user_gid);
+	}
 }
 
 /*
  * update_files - update the lastlog and faillog files
  */
-
 static void update_files (void)
 {
 	struct lastlog ll;
@@ -1259,12 +1303,10 @@ static void update_files (void)
 	 * left alone in case the UID was shared. It doesn't hurt anything
 	 * to just leave it be.
 	 */
-
 	if ((fd = open (LASTLOG_FILE, O_RDWR)) != -1) {
 		lseek (fd, (off_t) user_id * sizeof ll, SEEK_SET);
 		if (read (fd, (char *) &ll, sizeof ll) == sizeof ll) {
-			lseek (fd, (off_t) user_newid * sizeof ll,
-			       SEEK_SET);
+			lseek (fd, (off_t) user_newid * sizeof ll, SEEK_SET);
 			write (fd, (char *) &ll, sizeof ll);
 		}
 		close (fd);
@@ -1273,12 +1315,10 @@ static void update_files (void)
 	/*
 	 * Relocate the "faillog" entries in the same manner.
 	 */
-
 	if ((fd = open (FAILLOG_FILE, O_RDWR)) != -1) {
 		lseek (fd, (off_t) user_id * sizeof fl, SEEK_SET);
 		if (read (fd, (char *) &fl, sizeof fl) == sizeof fl) {
-			lseek (fd, (off_t) user_newid * sizeof fl,
-			       SEEK_SET);
+			lseek (fd, (off_t) user_newid * sizeof fl, SEEK_SET);
 			write (fd, (char *) &fl, sizeof fl);
 		}
 		close (fd);
@@ -1314,7 +1354,6 @@ static void move_mailbox (void)
 	 * replacing /var/spool/mail/luser with a hard link to /etc/passwd
 	 * between stat and chown).  --marekm
 	 */
-
 	snprintf (mailfile, sizeof mailfile, "%s/%s", maildir, user_name);
 	fd = open (mailfile, O_RDONLY | O_NONBLOCK, 0);
 	if (fd < 0) {
@@ -1335,16 +1374,34 @@ static void move_mailbox (void)
 		close (fd);
 		return;
 	}
-	if (uflg && fchown (fd, user_newid, (gid_t) - 1) < 0)
-		perror (_("failed to change mailbox owner"));
+	if (uflg) {
+		if (fchown (fd, user_newid, (gid_t) - 1) < 0) {
+			perror (_("failed to change mailbox owner"));
+		}
+#ifdef WITH_AUDIT
+		else {
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "changing mail file owner", user_newname,
+				      user_newid, 1);
+		}
+#endif
+	}
 
 	close (fd);
 
 	if (lflg) {
 		snprintf (newmailfile, sizeof newmailfile, "%s/%s",
 			  maildir, user_newname);
-		if (link (mailfile, newmailfile) || unlink (mailfile))
+		if (link (mailfile, newmailfile) || unlink (mailfile)) {
 			perror (_("failed to rename mailbox"));
+		}
+#ifdef WITH_AUDIT
+		else {
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "changing mail file name", user_newname,
+				      user_newid, 1);
+		}
+#endif
 	}
 }
 #endif
@@ -1359,7 +1416,6 @@ static struct pam_conv conv = {
 /*
  * main - usermod command
  */
-
 int main (int argc, char **argv)
 {
 	int grp_err = 0;
@@ -1370,9 +1426,10 @@ int main (int argc, char **argv)
 	int retval;
 #endif
 
-	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
-	user_groups = malloc ((1 + sys_ngroups) * sizeof (char *));
-	user_groups[0] = (char *) 0;
+#ifdef WITH_AUDIT
+	audit_help_open ();
+#endif
+
 	/*
 	 * Get my name so that I can use it to report errors.
 	 */
@@ -1382,6 +1439,19 @@ int main (int argc, char **argv)
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
+	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
+	user_groups = malloc ((1 + sys_ngroups) * sizeof (char *));
+	user_groups[0] = (char *) 0;
+
+	OPENLOG ("usermod");
+
+	is_shadow_pwd = spw_file_present ();
+#ifdef SHADOWGRP
+	is_shadow_grp = sgr_file_present ();
+#endif
+
+	process_flags (argc, argv);
+
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
@@ -1391,8 +1461,7 @@ int main (int argc, char **argv)
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval =
-		    pam_start ("shadow", pampw->pw_name, &conv, &pamh);
+		retval = pam_start ("usermod", pampw->pw_name, &conv, &pamh);
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -1410,43 +1479,15 @@ int main (int argc, char **argv)
 	}
 
 	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM authentication failed\n"),
-			 Prog);
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (1);
 	}
 #endif				/* USE_PAM */
 
-	OPENLOG (Prog);
-
-#ifdef SHADOWPWD
-	is_shadow_pwd = spw_file_present ();
-#endif
-#ifdef SHADOWGRP
-	is_shadow_grp = sgr_file_present ();
-#endif
-
-	/*
-	 * The open routines for the NDBM files don't use read-write as the
-	 * mode, so we have to clue them in.
-	 */
-
-#ifdef	NDBM
-	pw_dbm_mode = O_RDWR;
-#ifdef	SHADOWPWD
-	sp_dbm_mode = O_RDWR;
-#endif
-	gr_dbm_mode = O_RDWR;
-#ifdef	SHADOWGRP
-	sg_dbm_mode = O_RDWR;
-#endif
-#endif				/* NDBM */
-	process_flags (argc, argv);
-
 	/*
 	 * Do the hard stuff - open the files, change the user entries,
 	 * change the home directory, then close and update the files.
 	 */
-
 	open_files ();
 
 	usr_update ();
@@ -1473,7 +1514,6 @@ int main (int argc, char **argv)
 		 * Change the UID on all of the files owned by `user_id' to
 		 * `user_newid' in the user's home directory.
 		 */
-
 		chown_tree (dflg ? user_newhome : user_home,
 			    user_id, user_newid,
 			    user_gid, gflg ? user_newgid : user_gid);
@@ -1483,18 +1523,6 @@ int main (int argc, char **argv)
 		exit (E_GRP_UPDATE);
 
 #ifdef USE_PAM
-	if (retval == PAM_SUCCESS) {
-		retval = pam_chauthtok (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
-	}
-
-	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM chauthtok failed\n"), Prog);
-		exit (1);
-	}
-
 	if (retval == PAM_SUCCESS)
 		pam_end (pamh, PAM_SUCCESS);
 #endif				/* USE_PAM */
