@@ -22,23 +22,26 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: vipw.c,v 1.6 2003/06/19 18:11:01 kloczek Exp $")
-#include "defines.h"
+#ident "$Id: vipw.c,v 1.17 2005/09/07 15:00:45 kloczek Exp $"
+
 #include <errno.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 #include <sys/types.h>
-#include <signal.h>
+#include <unistd.h>
 #include <utime.h>
+#include "defines.h"
+#include "groupio.h"
+#include "nscd.h"
 #include "prototypes.h"
 #include "pwio.h"
-#include "shadowio.h"
-#include "groupio.h"
 #include "sgroupio.h"
-#include "nscd.h"
+#include "shadowio.h"
+/*
+ * Global variables
+ */
 static const char *progname, *filename, *fileeditname;
 static int filelocked = 0, createedit = 0;
 static int (*unlock) (void);
@@ -48,8 +51,7 @@ static int create_backup_file (FILE *, const char *, struct stat *);
 static void vipwexit (const char *, int, int);
 static void vipwedit (const char *, int (*)(void), int (*)(void));
 
-static int
-create_backup_file (FILE * fp, const char *backup, struct stat *sb)
+static int create_backup_file (FILE * fp, const char *backup, struct stat *sb)
 {
 	struct utimbuf ub;
 	FILE *bkfp;
@@ -62,13 +64,13 @@ create_backup_file (FILE * fp, const char *backup, struct stat *sb)
 	if (!bkfp)
 		return -1;
 
-	rewind (fp);
-	while ((c = getc (fp)) != EOF) {
-		if (putc (c, bkfp) == EOF)
-			break;
-	}
-
-	if (c != EOF || fflush (bkfp)) {
+	c = 0;
+	if (fseeko (fp, 0, SEEK_SET) == 0)
+		while ((c = getc (fp)) != EOF) {
+			if (putc (c, bkfp) == EOF)
+				break;
+		}
+	if (c != EOF || ferror (fp) || fflush (bkfp)) {
 		fclose (bkfp);
 		unlink (backup);
 		return -1;
@@ -94,10 +96,10 @@ static void vipwexit (const char *msg, int syserr, int ret)
 {
 	int err = errno;
 
-	if (filelocked)
-		(*unlock) ();
 	if (createedit)
 		unlink (fileeditname);
+	if (filelocked)
+		(*unlock) ();
 	if (msg)
 		fprintf (stderr, "%s: %s", progname, msg);
 	if (syserr)
@@ -111,8 +113,7 @@ static void vipwexit (const char *msg, int syserr, int ret)
 #endif
 
 static void
-vipwedit (const char *file, int (*file_lock) (void),
-	  int (*file_unlock) (void))
+vipwedit (const char *file, int (*file_lock) (void), int (*file_unlock) (void))
 {
 	const char *editor;
 	pid_t pid;
@@ -155,9 +156,7 @@ vipwedit (const char *file, int (*file_lock) (void),
 		   command line args in the EDITOR and VISUAL environment vars */
 		char *buf;
 
-		buf =
-		    (char *) malloc (strlen (editor) + strlen (fileedit) +
-				     2);
+		buf = (char *) malloc (strlen (editor) + strlen (fileedit) + 2);
 		snprintf (buf, strlen (editor) + strlen (fileedit) + 2,
 			  "%s %s", editor, fileedit);
 		if (system (buf) != 0) {
@@ -190,7 +189,6 @@ vipwedit (const char *file, int (*file_lock) (void),
 	 * ask the user what to do (edit again, save changes anyway, or quit
 	 * without saving). Use pwck or grpck to do the check.  --marekm
 	 */
-
 	createedit = 0;
 	unlink (filebackup);
 	link (file, filebackup);
@@ -205,7 +203,6 @@ vipwedit (const char *file, int (*file_lock) (void),
 	(*file_unlock) ();
 }
 
-
 int main (int argc, char **argv)
 {
 	int flag;
@@ -223,17 +220,17 @@ int main (int argc, char **argv)
 
 	while ((flag = getopt (argc, argv, "ghps")) != EOF) {
 		switch (flag) {
-		case 'p':
-			do_vipw = 1;
-			break;
 		case 'g':
 			do_vipw = 0;
 			break;
+		case 'h':
+			e = 0;
+		case 'p':
+			do_vipw = 1;
+			break;
 		case 's':
 			editshadow = 1;
 			break;
-		case 'h':
-			e = 0;
 		default:
 			printf (_("Usage:\n\
 `vipw' edits /etc/passwd        `vipw -s' edits /etc/shadow\n\
@@ -244,11 +241,9 @@ int main (int argc, char **argv)
 	}
 
 	if (do_vipw) {
-#ifdef SHADOWPWD
 		if (editshadow)
 			vipwedit (SHADOW_FILE, spw_lock, spw_unlock);
 		else
-#endif
 			vipwedit (PASSWD_FILE, pw_lock, pw_unlock);
 	} else {
 #ifdef SHADOWGRP
@@ -261,8 +256,6 @@ int main (int argc, char **argv)
 
 	nscd_flush_cache ("passwd");
 	nscd_flush_cache ("group");
-#ifdef SHADOWPWD
-	nscd_flush_cache ("shadow");
-#endif
+
 	return 0;
 }
